SHADER version 1
@OpenGL2.Vertex
#version 400
#define MAX_INSTANCES 256

//Uniforms
uniform mat4 cameramatrix;
uniform mat4 camerainversematrix;
uniform mat4 projectioncameramatrix;
uniform instancematrices { mat4 matrix[MAX_INSTANCES];} entity;
uniform vec3 cameraposition;

//Attributes
in vec3 vertex_position;
in vec2 vertex_texcoords0;
in vec3 vertex_normal;
in vec3 vertex_tangent;
in vec3 vertex_binormal;

//Outputs
out vec4 ex_vertexposition;
out vec2 ex_texcoords0;
out vec3 ex_normal;
out vec3 ex_tangent;
out vec3 ex_binormal;
out vec3 eyemat;

void main()
{

	//Entity matrix
	mat4 entitymatrix = entity.matrix[gl_InstanceID];
	mat4 entitymatrix_=entitymatrix;
	entitymatrix_[0][3]=0.0;
	entitymatrix_[1][3]=0.0;
	entitymatrix_[2][3]=0.0;
	entitymatrix_[3][3]=1.0;
	
	//Vertex position
	ex_vertexposition = entitymatrix_ * vec4(vertex_position, 1.0);
	gl_Position = projectioncameramatrix * ex_vertexposition;
	
	//Normals
	mat3 nmat = mat3(entitymatrix);
	ex_normal = (nmat * vertex_normal);	
	ex_tangent = normalize(nmat * vertex_tangent);
	ex_binormal = normalize(nmat*vertex_binormal);

	//Texture coordinates
	ex_texcoords0 = vertex_texcoords0;

	//Parallax tangent/binormal/normal matrix
	mat3 TBN_Matrix;
	TBN_Matrix[0] = ex_tangent*mat3(cameramatrix);
	TBN_Matrix[1] = ex_binormal*mat3(cameramatrix);
	TBN_Matrix[2] = ex_normal*mat3(cameramatrix);
	TBN_Matrix[2][2] -= 0.25; //lateral reduction
	vec3 inversecampos = vec3(camerainversematrix * ex_vertexposition);
	eyemat = vec3(inversecampos) * TBN_Matrix;
}
@OpenGLES2.Vertex

@OpenGLES2.Fragment

@OpenGL4.Vertex
#version 400
#define MAX_INSTANCES 256

//Uniforms
uniform mat4 cameramatrix;
uniform mat4 camerainversematrix;
uniform mat4 projectioncameramatrix;
uniform instancematrices { mat4 matrix[MAX_INSTANCES];} entity;
uniform vec3 cameraposition;

//Attributes
in vec3 vertex_position;
in vec2 vertex_texcoords0;
in vec3 vertex_normal;
in vec3 vertex_tangent;
in vec3 vertex_binormal;

//Outputs
out vec4 ex_vertexposition;
out vec2 ex_texcoords0;
out vec3 ex_normal;
out vec3 ex_tangent;
out vec3 ex_binormal;
out vec3 eyemat;

void main()
{

	//Entity matrix
	mat4 entitymatrix = entity.matrix[gl_InstanceID];
	mat4 entitymatrix_=entitymatrix;
	entitymatrix_[0][3]=0.0;
	entitymatrix_[1][3]=0.0;
	entitymatrix_[2][3]=0.0;
	entitymatrix_[3][3]=1.0;
	
	//Vertex position
	ex_vertexposition = entitymatrix_ * vec4(vertex_position, 1.0);
	gl_Position = projectioncameramatrix * ex_vertexposition;
	
	//Normals
	mat3 nmat = mat3(entitymatrix);
	ex_normal = (nmat * vertex_normal);	
	ex_tangent = normalize(nmat * vertex_tangent);
	ex_binormal = normalize(nmat*vertex_binormal);

	//Texture coordinates
	ex_texcoords0 = vertex_texcoords0;

	//Parallax tangent/binormal/normal matrix
	mat3 TBN_Matrix;
	TBN_Matrix[0] = ex_tangent*mat3(cameramatrix);
	TBN_Matrix[1] = ex_binormal*mat3(cameramatrix);
	TBN_Matrix[2] = ex_normal*mat3(cameramatrix);
	TBN_Matrix[2][2] -= 0.25; //lateral reduction
	vec3 inversecampos = vec3(camerainversematrix * ex_vertexposition);
	eyemat = vec3(inversecampos) * TBN_Matrix;
}
@OpenGL4.Fragment
#version 400

// Variables
float texture_size = 0.15;
float normal_texsize = 0.7;

//Surface
float surface_texsize = 0.4;
float surface_flaws = 0.5;

//Parallax
int parallax_steps = 10; 
float parallax_depth = 0.06;

//Refraction
float amplitude = 0.4;
float normal_smoothing = 7.0;
float contrast = 0.9; //color

//Cubemap
float cube_fresnel = 0.2;
float cube_brightness = 1.0;

//Blinn-Phong
vec3 lightposition = vec3(0.4,0.4,-0.6);
vec3 light_color = vec3(1,1,1);
float shininess = 456.0;

//Uniforms	
uniform sampler2D texture0;//diffuse
uniform sampler2D texture1;//normal
uniform sampler2D texture2;//specular
uniform sampler2D texture3;//height
uniform sampler2D texture4;//refraction tex
uniform samplerCube texture7;//cubemap
uniform vec3 cameraposition;
uniform mat4 projectionmatrix;

//Inputs
in vec4 ex_vertexposition;
in vec2 ex_texcoords0;
in vec3 ex_normal;
in vec3 ex_tangent;
in vec3 ex_binormal;
in vec3 eyemat;

out vec4 fragData0;
out vec4 fragData1;
out vec4 fragData2;

float blendLighten(float base, float blend) {
	return max(blend,base);
}

vec3 blendLighten(vec3 base, vec3 blend) {
	return vec3(blendLighten(base.r,blend.r),blendLighten(base.g,blend.g),blendLighten(base.b,blend.b));
}

vec3 blendLighten(vec3 base, vec3 blend, float opacity) {
	return (blendLighten(base, blend) * opacity + base * (1.0 - opacity));
}

void main()
{
	
	//Texturecoords
	vec2 tex = ex_texcoords0 * texture_size;

	//Diffuse
	vec4 outcolor = texture(texture0,ex_texcoords0 * surface_texsize) * 1.2;

	//Parallax
	//----------------------------------------------------------------------------
	float height;
	vec3 parallaxed_texcoords0 = vec3(tex, 1.0);
	vec3 dir = (-eyemat / eyemat.z)/parallax_steps; 
	dir.xy *= parallax_depth;

	for (int i = 0; i < parallax_steps; i++) {
		parallaxed_texcoords0 += dir;

		height = vec4(1).r - smoothstep(0.3,1.0, 1.0-texture(texture3, parallaxed_texcoords0.xy).r); 
		if (2.0*parallaxed_texcoords0.z < height) break;
	}

	for (int i = 0; i < parallax_steps; i++) {
		dir *= 0.5;
		if (2.0*parallaxed_texcoords0.z < height)
			parallaxed_texcoords0 -= dir;
		else
			parallaxed_texcoords0 += dir;
			height = vec4(1).r - smoothstep(0.3,1.0, 1.0-texture(texture3, parallaxed_texcoords0.xy).r); 
	}
	//-----------------------------------------------------------------------------
	
	//Normal
	vec3 normal = ex_normal;
	normal = normalize(texture(texture1,ex_texcoords0 * normal_texsize).xyz * 2.0 - 1.0);
	normal.z *= normal_smoothing;
	normal = ex_tangent * normal.x + ex_binormal * normal.y + ex_normal * normal.z;	
	normal = normalize(normal);

	//Cubemap
	vec3 cube_eye = normalize(ex_vertexposition.xyz - cameraposition);
	vec3 reflection = normalize(reflect(cube_eye, -normal)); 
	vec4 cube = clamp(texture(texture7,reflection),0,1) * cube_brightness;
	float ndotv = pow(max(0,dot(normal,reflection))*0.5+0.5,cube_fresnel);

	//Blinn-Phong
	vec3 lightvec = normalize(lightposition - projectionmatrix[3].xyz);
	vec3 eyevec = normalize(cameraposition - ex_vertexposition.xyz);
	vec3 halfvec = normalize(lightvec + eyevec);
	float ndoth = pow(max(0.0,dot(normal, halfvec)*0.5+0.5), shininess);

	//Refraction
	vec2 n1 = normal.xy * amplitude;
	vec2 n2 = ex_normal.xy * amplitude;
	vec4 refractcol = texture(texture4, parallaxed_texcoords0.xy + n1 - n2); //where ice effect happens
	refractcol = mix(refractcol,refractcol*dot(refractcol.xyz,vec3(0.57)) - 0.57,contrast);
	vec3 ice = blendLighten(refractcol.xyz,outcolor.xyz,surface_flaws);

	//Diffuse Output
	fragData0 = mix(cube,vec4(ice,1.0),ndotv);
	fragData0.xyz = mix(fragData0.xyz,light_color,ndoth);
	fragData0.a = 1.0;

	//Normals Output
	fragData1 = vec4(normalize(normal)*0.5+0.5,fragData0.a);
	fragData1.a = 1/255.0;
}
